<?php

namespace Kartographer\Tag;

use FormatJson;
use Html;
use Kartographer\CoordFormatter;
use Kartographer\SpecialMap;

/**
 * The <maplink> tag creates a link that, when clicked,
 */
class MapLink extends TagHandler {

	public const TAG = 'maplink';

	/** @var string */
	private $cssClass = '';

	/**
	 * @inheritDoc
	 */
	protected function parseArgs(): void {
		$this->state->useMaplink();
		parent::parseArgs();
		$this->cssClass = $this->getText( 'class', '', '/^(|[a-zA-Z][-_a-zA-Z0-9]*)$/' );
	}

	/**
	 * @inheritDoc
	 */
	protected function render(): string {
		$parserOutput = $this->parser->getOutput();
		$parserOutput->addModules( [ 'ext.kartographer.link' ] );

		// @todo: Mapbox markers don't support localized numbers yet
		$text = $this->getText( 'text', null, '/\S+/' );
		if ( $text === null ) {
			$text = $this->counter
				?: CoordFormatter::format( $this->lat, $this->lon, $this->getLanguage() );
		}
		$text = $this->parser->recursiveTagParse( $text, $this->frame );

		$attrs = [
			'class' => 'mw-kartographer-maplink',
			'data-mw' => 'interface',
			'data-style' => $this->mapStyle,
			'href' => SpecialMap::link( $this->lat, $this->lon, $this->zoom, $this->resolvedLangCode )
				->getLocalURL()
		];

		if ( $this->zoom !== null ) {
			$attrs['data-zoom'] = $this->zoom;
		}
		if ( $this->lat !== null && $this->lon !== null ) {
			$attrs['data-lat'] = $this->lat;
			$attrs['data-lon'] = $this->lon;
		}
		if ( $this->specifiedLangCode !== null ) {
			$attrs['data-lang'] = $this->specifiedLangCode;
		}
		$style = $this->extractMarkerCss();
		if ( $style ) {
			$attrs['class'] .= ' mw-kartographer-autostyled';
			$attrs['style'] = $style;
		}
		if ( $this->cssClass !== '' ) {
			$attrs['class'] .= ' ' . $this->cssClass;
		}
		if ( $this->showGroups ) {
			$attrs['data-overlays'] = FormatJson::encode( $this->showGroups, false,
				FormatJson::ALL_OK );
		}
		return Html::rawElement( 'a', $attrs, $text );
	}

	/**
	 * Extracts CSS style to be used by the link from GeoJSON
	 * @return string
	 */
	private function extractMarkerCss(): string {
		if ( $this->config->get( 'KartographerUseMarkerStyle' )
			&& $this->markerProperties
			&& isset( $this->markerProperties->{'marker-color'} )
			// JsonSchema already validates this value for us, however this regex will also fail
			// if the color is invalid
			&& preg_match( '/^#?((?:[\da-f]{3}){1,2})$/i', $this->markerProperties->{'marker-color'}, $m )
		) {
			return "background: #{$m[1]};";
		}

		return '';
	}
}
